/*
 * Copyright (c) 2009 Lukas Mejdrech
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * - Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * - The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/** @addtogroup ip
 *  @{
 */

/**
 * @file
 */

#include <errno.h>

#include <sys/types.h>

#include "../../include/ip_client.h"

#include "../../structures/packet/packet.h"
#include "../../structures/packet/packet_client.h"

#include "ip_header.h"

int ip_client_prepare_packet( packet_t packet, int protocol, int ttl, int tos, int dont_fragment, int ipopt_length ){
	ip_header_ref	header;
	uint8_t *		data;
	int				padding;

	padding =  ipopt_length % 4;
	if( padding ){
		padding = 4 - padding;
		ipopt_length += padding;
	}
	data = ( uint8_t * ) packet_prefix( packet, sizeof( ip_header_t ) + padding );
	if( ! data ) return ENOMEM;
	while( padding -- ) data[ sizeof( ip_header_t ) + padding ] = IPOPT_NOOP;
	header = ( ip_header_ref ) data;
	header->ihl = ( sizeof( ip_header_t ) + ipopt_length ) / 4;
	header->ttl = ( ttl > 0 ) ? (( ttl <= MAXTTL ) ? ttl : MAXTTL ) : IPDEFTTL;
	header->tos = tos;
	header->protocol = protocol;
	if( dont_fragment ) header->flags = IPFLAG_DONT_FRAGMENT;
	return EOK;
}

/** @}
 */
